@@ -22,7 +22,8 @@
 import re
 import unidecode
 
-from majormode.perseus.constant import area
+from majormode.perseus.constant.area import AREA_LEVEL_COUNTRY
+from majormode.perseus.constant.area import AREA_LEVEL_WARD
 from majormode.perseus.model.label import Label
 from majormode.perseus.model.geolocation import BoundingBox
 from majormode.perseus.model.geolocation import GeoPoint
@@ -159,7 +160,8 @@ class AreaService(BaseRdbmsService):
                     content,
                     get_area_path(area_id, p_locale:=%(locale)s) AS path,
                     locale
-                  FROM find_areas_with_ip_address(
+                  FROM 
+                    find_areas_with_ip_address(
                       %(ip_address)s,
                       p_lowest_area_level:=%(lowest_area_level)s,
                       p_highest_area_level:=%(highest_area_level)s,
@@ -178,7 +180,72 @@ class AreaService(BaseRdbmsService):
                     'locale': Locale,
                     'parent_area_id': cast.string_to_uuid
                 })
-                for row in cursor.fetch_all()]
+                for row in cursor.fetch_all()
+            ]
+
+            for area in areas:
+                area.label = Label(area.content, area.locale)
+                area.path = Label(area.path, area.locale)
+
+                del area.content, area.path, area.locale
+
+                area.centroid = GeoPoint(area.latitude, area.longitude)
+                del area.longitude, area.latitude
+
+                area.bounding_box = BoundingBox(
+                    GeoPoint(area.y_min, area.x_min),
+                    GeoPoint(area.y_max, area.x_max))
+                del area.x_min, area.y_min, area.x_max, area.y_max
+
+            return areas
+
+    def find_areas_with_location(
+            self,
+            location,
+            connection=None,
+            lowest_area_level=AREA_LEVEL_WARD,
+            highest_area_level=AREA_LEVEL_COUNTRY,
+            locale=None):
+        with self.acquire_rdbms_connection(auto_commit=False, connection=connection) as connection:
+            cursor = connection.execute(
+                """
+                SELECT 
+                    area_id,
+                    parent_area_id,
+                    ST_X(centroid) AS longitude,
+                    ST_Y(centroid) AS latitude,
+                    ST_XMin(bounding_box) AS x_min,
+                    ST_YMin(bounding_box) AS y_min,
+                    ST_XMax(bounding_box) AS x_max,
+                    ST_YMax(bounding_box) AS y_max,
+                    area_type,
+                    area_level,
+                    content,
+                    get_area_path(area_id, p_locale:=%(locale)s) AS path,
+                    locale
+                  FROM 
+                    find_areas_with_location(
+                      ST_SetSRID(ST_MakePoint(%(longitude)s, %(latitude)s), 4326),
+                      p_lowest_area_level:=%(lowest_area_level)s,
+                      p_highest_area_level:=%(highest_area_level)s,
+                      p_locale:=%(locale)s)
+                """,
+                {
+                    'highest_area_level': highest_area_level,
+                    'latitude': location.latitude,
+                    'locale': locale,
+                    'longitude': location.longitude,
+                    'lowest_area_level': lowest_area_level
+                })
+
+            areas = [
+                row.get_object({
+                    'area_id': cast.string_to_uuid,
+                    'locale': Locale,
+                    'parent_area_id': cast.string_to_uuid
+                })
+                for row in cursor.fetch_all()
+            ]
 
             for area in areas:
                 area.label = Label(area.content, area.locale)
@@ -537,8 +604,8 @@ class AreaService(BaseRdbmsService):
             self,
             ip_address,
             connection=None,
-            lowest_area_level=area.AREA_LEVEL_WARD,
-            highest_area_level=area.AREA_LEVEL_COUNTRY,
+            lowest_area_level=AREA_LEVEL_WARD,
+            highest_area_level=AREA_LEVEL_COUNTRY,
             minimal_area_surface=None,
             locale=DEFAULT_LOCALE):
         """
@@ -753,8 +820,8 @@ class AreaService(BaseRdbmsService):
             self,
             location,
             connection=None,
-            lowest_area_level=area.AREA_LEVEL_WARD,
-            highest_area_level=area.AREA_LEVEL_COUNTRY,
+            lowest_area_level=AREA_LEVEL_WARD,
+            highest_area_level=AREA_LEVEL_COUNTRY,
             locale=None,
             minimal_area_surface=None):
         """
